<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All rights reserved.
 *
 * Author: Chandravadana Kandregula
 * Code Release Version: 1.0.0
 *
 * Class Data
 * @module Pinpoint_DispatchTimer;
 */

namespace Pinpoint\DispatchTimer\Model;

use Magento\Catalog\Helper\Data as catalogHelper;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;
use Pinpoint\DispatchTimer\Helper\Data as DataHelper;

class ShippingMessage
{
    private $timezone;

    private $helper;
    /**
     * @var catalogHelper
     */
    private $catalogHelper;

    /**
     * Data constructor.
     * @param TimezoneInterface $timezone
     * @param DataHelper $helper
     * @param catalogHelper $catalogHelper
     */
    public function __construct(
        TimezoneInterface $timezone,
        DataHelper $helper,
        catalogHelper $catalogHelper
    ) {
        $this->timezone = $timezone;
        $this->helper = $helper;
        $this->catalogHelper = $catalogHelper;
    }

    /**
     * @param $sku
     * @param $message
     * @return array
     */
    public function getShippingData($sku, $message)
    {
        $dispatchTimerStatus = $this->helper->getDispatchTimerStatus();
        if (!$dispatchTimerStatus) {
            return ['status' => false];
        }

        $daysToShip = $this->helper->getDaysToShip();
        $dispatchDays = $this->helper->getDispatchDays();
        $daysOfWeek = $this->helper->getDaysOfWeek();
        $holidays = $this->helper->getHolidays();
        $cutOffDateTime = $this->helper->getCutOffDateTime();
        $cutOffTime = $this->helper->getCutOffTimeFormatted();
        $shippingMessage = $this->helper->getShippingMessage();

        $currentDateTime = $this->timezone->date()->format('Y-m-d H:i:s');
        $currentTime = $this->timezone->date()->format('H:i:s');
        $shippingDateTime = '';

        if (strtotime($cutOffTime) <= strtotime($currentTime)) {
            $shippingTime = date("Y-m-d H:i:s", strtotime(date("Y-m-d H:i:s", strtotime($cutOffTime))));
        } else {
            $shippingTime = date("Y-m-d H:i:s", strtotime(date("H:i:s", strtotime($cutOffTime))));
        }
        $shippingTime = $this->checkHoliday($shippingTime, $holidays, $daysOfWeek, $dispatchDays);
        if (strtotime($cutOffDateTime) <= strtotime($currentDateTime)) {
            $shippingDateTime = date("Y-m-d H:i:s", strtotime(date("Y-m-d H:i:s", strtotime($cutOffDateTime)) . "+1 day"));
        } else {
            $shippingDateTime = $cutOffDateTime;
        }

        $shippingDateTime  = $this->checkHoliday($shippingDateTime, $holidays, $daysOfWeek, $dispatchDays);

        if ($daysToShip) {
            $days = 1;
            while ($daysToShip >= $days) {
                $shippingDateTime = date("Y-m-d H:i:s", strtotime(date("Y-m-d H:i:s", strtotime($shippingDateTime)) . "+1 day"));
                $shippingDateTime  = $this->checkHoliday($shippingDateTime, $holidays, $daysOfWeek, $dispatchDays);
                $days++;
            }
        }
        $formattedShippingDateTime = date('l jS F', strtotime($shippingDateTime));

        $standardMessage = $this->getStandardMessage($sku, $message);
        $noMessage = $this->getNoMessage($sku);

        return [
            'status' => true,
            'shipping_date_time' => $formattedShippingDateTime,
            'dispatch_time' => $shippingTime,
            'message' => $shippingMessage,
            'standard_message' => $standardMessage,
            'nomessage_flag' => $noMessage,

        ];
    }

    /**
     * @param $shippingDateTime
     * @param $holidays
     * @param $daysOfWeek
     * @param $dispatchDays
     * @return false|string
     */
    public function checkHoliday($shippingDateTime, $holidays, $daysOfWeek, $dispatchDays)
    {
        $search = true;

        while ($search) {
            $dispatch_day = ucwords(date("l", strtotime($shippingDateTime)));

            if (!in_array($shippingDateTime, $holidays) && in_array(array_search($dispatch_day, $daysOfWeek), $dispatchDays)) {
                $search = false;
            } else {
                $shippingDateTime = date("Y-m-d H:i:s", strtotime(date("Y-m-d H:i:s", strtotime($shippingDateTime)) . "+1 day"));
            }
        }

        return $shippingDateTime;
    }

    /**
     * @param $sku
     * @param $message
     * @return mixed|string
     */
    public function getStandardMessage($sku, $message)
    {
        if (isset($message) && $message !== '') {
            return $message;
        }
        if (in_array($sku, $this->helper->getStandMessageSkus())) {
            return $this->helper->getStandardMessage();
        }
        return false;
    }

    /**
     * @param $sku
     * @return bool
     */
    public function getNoMessage($sku)
    {
        if (in_array($sku, $this->helper->getSkipSkus())) {
            return true;
        }
        return false;
    }
}
