<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All rights reserved.
 *
 * Author: Chandravadana Kandregula
 * Code Release Version: 1.0.0
 *
 * Class UpgradeData
 * @package Pinpoint\ProductAddons\Setup;
 */

namespace Pinpoint\DispatchTimer\Setup;

use Magento\Catalog\Model\Product;
use Magento\Eav\Setup\EavSetup;
use Magento\Eav\Setup\EavSetupFactory;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\UpgradeDataInterface;
use Pinpoint\DispatchTimer\Helper\Data;
use Zend_Validate_Exception;

class UpgradeData implements UpgradeDataInterface
{
    /**
     * @var EavSetupFactory
     */
    private $eavSetupFactory;
    /**
     * @var Data
     */
    private $helper;

    /**
     * InstallData constructor.
     * @param EavSetupFactory $eavSetupFactory
     * @param Data $helper
     */
    public function __construct(EavSetupFactory $eavSetupFactory, Data $helper)
    {
        $this->eavSetupFactory = $eavSetupFactory;
        $this->helper = $helper;
    }

    /**
     * @param ModuleDataSetupInterface $setup
     * @param ModuleContextInterface $context
     * @throws LocalizedException
     * @throws Zend_Validate_Exception
     */
    public function upgrade(ModuleDataSetupInterface $setup, ModuleContextInterface $context)
    {
        $setup->startSetup();

        /** @var EavSetup $eavSetup */
        $eavSetup = $this->eavSetupFactory->create(['setup' => $setup]);
        if (version_compare($context->getVersion(), '0.1.1', '<')) {
            $eavSetup->removeAttribute(Product::ENTITY, DATA::SHIPPING_MESSAGE);
            $eavSetup->addAttribute(
                Product::ENTITY,
                DATA::SHIPPING_MESSAGE,
                array_merge(
                    $this->getTextAttributeOptions(),
                    ['label' => $this->helper->getLabel()],
                    ['note' => __('If empty, shipping message will be displayed from Dispatch Timer Module Settings')]

                )
            );
        }

        $setup->endSetup();
    }

    /**
     * @return array
     */
    private function getTextAttributeOptions()
    {
        return [
            'type' => 'varchar',
            'input' => 'text',
            'group' => 'General',
            'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_WEBSITE,
            'visible' =>  true,
            'required' => false,
            'user_defined' => true,
            'searchable' => false,
            'filterable' => false,
            'comparable' => false,
            'visible_on_front' => false,
            'used_in_product_listing' => false,
            'unique' => false
        ];
    }
}
